'use strict'

const Logger = require('../logger')

/**
 * Handlebars Instrumentation
 * https://www.npmjs.com/package/handlebars
 * @param handlebars
 * @param tracer
 * @param shimmer
 * @param version
 */
module.exports = function (handlebars, tracer, shimmer, version) {
  Logger.debug('Handlebars version %s', version)

  shimmer.wrap(handlebars, 'compile', wrapCompile)
  return handlebars

  /**
   * @param original
   * @return {function(): *}
   */
  function wrapCompile (original) {
    return function () {
      const span = tracer.createSpan('handlebars.compile')
      if (span) {
        span.addProperty('CATEGORY', 'Template')
        span.addProperty('SUBCATEGORY', 'Compile')
      }

      var result = original.apply(this, arguments)
      if (span) {
        tracer.endSpan(span)
      }

      return typeof result === 'function' ? wrapRender(result) : result
    }
  }

  /**
   * @param original
   * @return {function(): *}
   */
  function wrapRender (original) {
    return function () {
      const span = tracer.createSpan('handlebars.render')
      if (span) {
        span.addProperty('CATEGORY', 'Template')
        span.addProperty('SUBCATEGORY', 'Render')
      }

      var result = original.apply(this, arguments)
      if (span) {
        tracer.endSpan(span)
      }

      return result
    }
  }
}
