'use strict'

const sprintf = require('sprintf-js').sprintf
const Logger = require('../logger')
const stringify = require('json-stringify-safe')
const Util = require('../util')
const Config = require('../config')

let tracer
let config
const INJECTION_CONTENT_TEMPLATE = '<script type="text/javascript">(window.StackifySettings || (window.StackifySettings = %s))</script><script src="%s" data-key="%s" async></script>'

function RumGenerator (_tracer) {
  if (_tracer) {
    tracer = _tracer
    config = tracer._config
  } else {
    config = Config
  }

  this._rumAssetUrl = null
  this.setRumAssetUrl(config.rumAssetUrl)
}

RumGenerator.prototype.getRumInjectionContent = function getRumInjectionContent () {
  let transaction = tracer.getActiveTransaction()

  if (!transaction) {
    Logger.debug('RumGenerator.getRumInjectionContent - There is no active transaction')
    return ''
  }

  let rumKey = config.getRumKey()
  let rumScriptUrl = config.getRumScriptUrl()

  if (!rumKey || !rumScriptUrl) {
    Logger.debug('RumGenerator.getRumInjectionContent - There is no rum key or rum script url')
    return ''
  }

  let settings = {
    'ID': transaction.id(),
    'Name': Buffer.from(config.getApplicationName()).toString('base64'),
    'Env': Buffer.from(config.getEnvironmentName()).toString('base64')
  }

  let rootSpan = transaction.rootSpan
  let reportingUrl = Util.getReportingUrl(rootSpan)
  if (reportingUrl) {
    reportingUrl = Util.maskReportingUrl(reportingUrl)

    let requestMethod = rootSpan && rootSpan.props && rootSpan.props['METHOD'] ? rootSpan.props['METHOD'] : null
    if (requestMethod) {
      reportingUrl = requestMethod + '-' + reportingUrl
    }

    settings['Trans'] = Buffer.from(reportingUrl).toString('base64')
  }

  // Set RUM Context on transaction
  transaction.setRum(true)

  return sprintf(
    INJECTION_CONTENT_TEMPLATE,
    stringify(settings),
    rumScriptUrl,
    rumKey
  )
}

RumGenerator.prototype.getRumAssetUrl = function getRumAssetUrl () {
  return this._rumAssetUrl
}

RumGenerator.prototype.setRumAssetUrl = function setRumAssetUrl (assetUrl) {
  this._rumAssetUrl = assetUrl
}
const rumGenerator = new RumGenerator()
rumGenerator.RumGenerator = RumGenerator

module.exports = rumGenerator
